@@ -20,6 +20,7 @@
 #include <plat/cache-aurora-l2.h>
 #include <asm/io.h>
 #include <mach/smp.h>
+#include <asm/cp15.h>
 #include "ctrlEnv/mvCtrlEnvSpec.h"
 #include "ctrlEnv/mvCtrlEnvLib.h"
 #include "ctrlEnv/mvSemaphore.h"
@@ -253,7 +254,9 @@ out:
 #define RANGE_OP
 
 static int l2_wt_override = 0;
+#ifdef CONFIG_AURORA_L2_OUTER
 static DEFINE_SPINLOCK(smp_l2cache_lock);
+#endif
 
 /*
  * Low-level cache maintenance operations.
@@ -529,34 +532,6 @@ static void __init enable_cache(u32 mask)
 	set_cr(cr1 | mask);
 }
 
-#ifdef CONFIG_MV_AMP_ENABLE
-static bool is_l2_enabled(void)
-{
-	u32  reg;
-	bool enabled;
-
-	/* Read and return enable bit */
-	reg 	= readl(auroraL2_base + L2_CONTROL);
-	enabled = (reg & 0x1);
-
-	return enabled;
-}
-
-static void wait_l2_enabled(void)
-{
-	u32 usec_timeout = 1000;
-
-	while((is_l2_enabled() == 0) && (usec_timeout > 0))
-	{
-		udelay(1);
-		usec_timeout--;
-	}		
-
-	if(usec_timeout == 0)
-		printk(KERN_ERR "AMP: Waiting too long for L2 enable\n");
-}
-#endif /* CONFIG_MV_AMP_ENABLE */
-
 static void __init enable_l2(void)
 {
 	u32 u, mask;
@@ -572,25 +547,11 @@ static void __init enable_l2(void)
 	__asm__ __volatile__("mcr p15, 1, %0, c15, c2, 0\n" : : "r" (u));
 
 	u = readl(auroraL2_base+L2_CONTROL);
-#ifdef CONFIG_MV_AMP_ENABLE
-	if ((!(u & 1)) || (is_primary_amp() == 0)) {
-#else
 	if (!(u & 1)) {
-#endif
 		printk(KERN_INFO "Aurora L2 Cache Enabled\n");
 		u |= 1;
 		mask = invalidate_and_disable_cache();
-
-#ifdef CONFIG_MV_AMP_ENABLE
-		mvHwBarrier(2); // Wait for all AMP groups YY - why 2 CPUs ??
-		
-		if(is_primary_amp())
-			writel(u, auroraL2_base + L2_CONTROL);
-		else
-			wait_l2_enabled();
-#else
 		writel(u, auroraL2_base+L2_CONTROL);
-#endif
 		enable_cache(mask);
 	}
 }
@@ -692,7 +653,7 @@ int __init aurora_l2_init(void __iomem *base)
 	res_file->read_proc = proc_auroraL2_counter_read;
 	res_file->write_proc = proc_auroraL2_counter_write;
 #endif /* CONFIG_CACHE_AURORAL2_EVENT_MONITOR_ENABLE */
-#endif	
+#endif
 
 #ifdef CONFIG_AURORA_L2_OUTER
 	outer_cache.inv_range = aurora_l2_inv_range;
@@ -706,10 +667,10 @@ int __init aurora_l2_init(void __iomem *base)
 	if(is_primary_amp())
 #endif
 	{
-	
+
 		/* 1. Write to AuroraL2 Auxiliary Control Register, 0x104
 		*    Setting up Associativity, Way Size, and Latencies
-		*/	
+		*/
 		aux = readl(auroraL2_base + L2_AUX_CTRL_REG);
 		aux &= ~L2ACR_REPLACEMENT_MASK;
 		aux |= l2rep;
@@ -718,25 +679,25 @@ int __init aurora_l2_init(void __iomem *base)
 		aux |= L2ACR_FORCE_WRITE_BACK_POLICY;
 #endif
 		writel(aux, auroraL2_base + L2_AUX_CTRL_REG);
-	
+
 		l2_wt_override = ((aux & (0x3)) == 0x2 ? 1:0);
 		/* 3. Secure write to AuroraL2 Invalidate by Way, 0x77c
-		*/ 
+		*/
 		auroraL2_inv_all();
-	
+
 		/* 4. Write to the Lockdown D and Lockdown I Register 9 if required
-		*/  
-	
-		/* 5. Write to interrupt clear register, 0x220, to clear any residual 
+		*/
+
+		/* 5. Write to interrupt clear register, 0x220, to clear any residual
 		*    raw interrupt set.
 		*/
 		writel(0x1FF, auroraL2_base + L2_INT_CAUSE_REG);
-		
+
 		/* 6. Enable L2 cache
 		*/
+		enable_l2();
 	}
 
-	enable_l2();
 	auroraL2_enable = 1;
 
 	return 0;
