@@ -11,7 +11,6 @@
 #include <linux/kernel.h>
 #include <linux/errno.h>
 #include <linux/smp.h>
-#include <linux/completion.h>
 
 #include <asm/cacheflush.h>
 #include <../cpuidle.h>
@@ -29,20 +28,17 @@
 
 extern volatile int pen_release;
 
-static DECLARE_COMPLETION(cpu_killed);
-
-
 static inline void platform_do_lowpower(unsigned int cpu)
 {
-	
+
 	/*
 	 * there is no power-control hardware on this platform, so all
 	 * we can do is put the core into WFI; this is safe as the calling
 	 * code will have already disabled interrupts
 	 */
-	
+
 	for (;;) {
-		
+
 		/*
 		 * here's the WFI
 		*/
@@ -83,30 +79,18 @@ int platform_cpu_kill(unsigned int cpu)
  */
 void __ref platform_cpu_die(unsigned int cpu)
 {
-#ifdef DEBUG
-	unsigned int this_cpu = hard_smp_processor_id();
-
-	if (cpu != this_cpu) {
-		printk(KERN_CRIT "Eek! platform_cpu_die running on %u, should be %u\n",
-			   this_cpu, cpu);
-		BUG();
-	}
-#endif
-
-	complete(&cpu_killed);
-
 	/*
 	 * we're ready for shutdown now, so do it
 	 */
 
 	flush_cache_all();
-
+#ifdef CONFIG_SHEEVA_DEEP_IDLE
 	armadaxp_fabric_prepare_hotplug();
-
+#endif
 	/* none zero means deepIdle wasn't entered and regret event happened */
 
 	platform_do_lowpower(cpu);
-	
+
 	/*
 	 * bring this CPU back into the world of cache
 	 * coherency, and then restore interrupts - will never get here it.
@@ -121,7 +105,7 @@ int platform_cpu_disable(unsigned int cpu)
 	 * we don't allow CPU 0 to be shutdown (it is still too special
 	 * e.g. clock tick interrupts)
 	 */
-	
+
 	return cpu == 0 ? -EPERM : 0;
 }
 
