@@ -334,6 +334,20 @@ struct dma_chan *dma_find_channel(enum dma_transaction_type tx_type)
 }
 EXPORT_SYMBOL(dma_find_channel);
 
+/*
+ * net_dma_find_channel - find a channel for net_dma
+ * net_dma has alignment requirements
+ */
+struct dma_chan *net_dma_find_channel(void)
+{
+	struct dma_chan *chan = dma_find_channel(DMA_MEMCPY);
+	if (chan && !is_dma_copy_aligned(chan->device, 1, 1, 1))
+		return NULL;
+
+	return chan;
+}
+EXPORT_SYMBOL(net_dma_find_channel);
+
 /**
  * dma_issue_pending_all - flush all pending operations across all channels
  */
@@ -512,8 +526,8 @@ struct dma_chan *__dma_request_channel(dma_cap_mask_t *mask, dma_filter_fn fn, v
 					 dma_chan_name(chan));
 				list_del_rcu(&device->global_node);
 			} else if (err)
-				pr_debug("dmaengine: failed to get %s: (%d)\n",
-					 dma_chan_name(chan), err);
+				pr_debug("%s: failed to get %s: (%d)\n",
+					__func__, dma_chan_name(chan), err);
 			else
 				break;
 			if (--device->privatecnt == 0)
@@ -567,7 +581,7 @@ void dmaengine_get(void)
 				break;
 			} else if (err)
 				pr_debug("%s: failed to get %s: (%d)\n",
-				       __func__, dma_chan_name(chan), err);
+					__func__, dma_chan_name(chan), err);
 		}
 	}
 
@@ -695,12 +709,12 @@ int dma_async_device_register(struct dma_device *device)
 		!device->device_prep_dma_interrupt);
 	BUG_ON(dma_has_cap(DMA_SG, device->cap_mask) &&
 		!device->device_prep_dma_sg);
-	BUG_ON(dma_has_cap(DMA_SLAVE, device->cap_mask) &&
-		!device->device_prep_slave_sg);
 	BUG_ON(dma_has_cap(DMA_CYCLIC, device->cap_mask) &&
 		!device->device_prep_dma_cyclic);
 	BUG_ON(dma_has_cap(DMA_SLAVE, device->cap_mask) &&
 		!device->device_control);
+	BUG_ON(dma_has_cap(DMA_INTERLEAVE, device->cap_mask) &&
+		!device->device_prep_interleaved_dma);
 
 	BUG_ON(!device->device_alloc_chan_resources);
 	BUG_ON(!device->device_free_chan_resources);
@@ -916,7 +930,7 @@ dma_async_memcpy_buf_to_pg(struct dma_chan *chan, struct page *page,
 	/*
 	  This code snippet is for Marvell XOR engine that supports operation on len < 128
 	  So if we get a copy operation smaller than 128, we use memcpy
-	  Also, we're creating a dummy dma operation in order to satisfy upper layers waiting 
+	  Also, we're creating a dummy dma operation in order to satisfy upper layers waiting
 	  for a valid cookie return code.
 	*/
 	if (len < DMA_ENGINE_MIN_OP_SIZE)
@@ -999,7 +1013,7 @@ dma_async_memcpy_pg_to_pg(struct dma_chan *chan, struct page *dest_pg,
 	/*
 	  This code snippet is for Marvell XOR engine that doesn't support operations on len < 128
 	  So if we get a copy operation smaller than 128, we use memcpy
-	  Also, we're creating a dummy dma operation in order to satisfy upper layers waiting 
+	  Also, we're creating a dummy dma operation in order to satisfy upper layers waiting
 	  for a valid cookie return code.
 	*/
 	if (len < DMA_ENGINE_MIN_OP_SIZE)
@@ -1007,7 +1021,7 @@ dma_async_memcpy_pg_to_pg(struct dma_chan *chan, struct page *dest_pg,
 		void * dst = kmap_atomic(dest_pg, KM_USER0) + dest_off;
 		memcpy(dst, src_pg+src_off, len);
 		kunmap_atomic(dst, KM_USER0);
-		
+
 		dma_src = dma_map_page(dev->dev, temp_page, 0, PAGE_SIZE, DMA_TO_DEVICE);
 		dma_dest = dma_map_page(dev->dev, temp_page, 0, PAGE_SIZE, DMA_FROM_DEVICE);
 		flags = DMA_CTRL_ACK;
